#!/bin/bash

# dyncnn is a deep-learning algorithm for the prediction of
# interacting object dynamics
#
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
# Written by Francois Fleuret <francois.fleuret@idiap.ch>
#
# This file is part of dyncnn.
#
# dyncnn is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# dyncnn is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dyncnn.  If not, see <http://www.gnu.org/licenses/>.

# This script creates the synthetic data-set for shape collision

set -e
set -o pipefail

[[ "${TORCH_NB_THREADS}" ]] || echo "You can set \$TORCH_NB_THREADS to the proper value (default 1)."
[[ "${TORCH_USE_GPU}" ]] || echo "You can set \$TORCH_USE_GPU to 'yes' or 'no' (default 'no')."
[[ "${DYNCNN_DATA_DIR}" ]] || DYNCNN_DATA_DIR="./data/10p-mg"
[[ "${DYNCNN_RESULT_DIR}" ]] || DYNCNN_RESULT_DIR="./results"

if [[ -d "${DYNCNN_DATA_DIR}" ]]; then
    echo "Found ${DYNCNN_DATA_DIR}, checking the number of images in there."
    if [[ $(find "${DYNCNN_DATA_DIR}" -name "dyn_*.png" | wc -l) == 150000 ]]; then
        echo "Looks good !"
    else
        echo "I do not find the proper number of images. Please remove the dir and re-run this scripts, or fix manually."
        exit 1
    fi
else
    # Creating the data-base
    make -j -k
    mkdir -p "${DYNCNN_DATA_DIR}"
    ./flatland 50000 \
               --every_nth 4 --nb_frames 5 \
               --multi_grasp --nb_shapes 10 \
               --dir "${DYNCNN_DATA_DIR}"
fi

# Train the model (takes days)

if [[ ! -f "${DYNCNN_RESULT_DIR}"/epoch_01000_model ]]; then
    ./dyncnn.lua --heavy --dataDir="${DYNCNN_DATA_DIR}" \
                 --resultFreq=100 \
                 --resultDir "${DYNCNN_RESULT_DIR}" \
                 --nbEpochs 1000
fi

# Create the images of internal activations

for n in 2 12; do
    ./dyncnn.lua --heavy --dataDir=./data/10p-mg/ \
                 --learningStateFile="${DYNCNN_RESULT_DIR}"/epoch_01000_model \
                 --resultDir="${DYNCNN_RESULT_DIR}" \
                 --noLog \
                 --exampleInternals=${n}
done
